from flask import Blueprint, request, jsonify, make_response, json
from app.gateways.models import Gateways
from app.zone.models import Zone
from app.rooms.models import Room
from app.gateways.schemas import gateway_schema
from app import db
from app.utils import token_required, delete_record, paginated_query, single_record_query

gateways = Blueprint('gateways', __name__)

@gateways.route('/api/v1/add/gateway', methods=['POST'])
@token_required(1, 2, 3)
def create_gateways(current_user):
    try:
        device_dict = json.loads(request.form.get('formData'))
        loaded_gateway_data = gateway_schema.load({
            'gateway_name': device_dict['gateway_name'].strip(),
            'gateway_mac_address': device_dict['gateway_mac_address'].strip(),
            'x_coordinate': device_dict['x_coordinate'] if device_dict['x_coordinate'] else 0,
            'y_coordinate': device_dict['y_coordinate'] if device_dict['y_coordinate'] else 0,
            'zone_id': device_dict['zone_id'],
            'room_id': device_dict['room_id'],
            'site_id': request.args.get('site_id'),
            'room_name': device_dict['room_name'],
        })
        loaded_gateway_data.pop("room_name")
        db.session.add(Gateways(**loaded_gateway_data))
        db.session.commit()     
        return make_response(jsonify({'message': 'Gateway added successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to add device', "status": "error"}), 500)

@gateways.route('/api/v1/get/gateway', methods=['GET'])
@token_required(1, 2, 3)
def get_gateways(current_user):
    try:
        gateways_query = Gateways.query.order_by(Gateways.gateway_name.asc())
        gateways_paginated, data= paginated_query(gateways_query,
            [Gateways.gateway_mac_address, Gateways.gateway_name],
                        Gateways.site_id)
        serialized_gateways = []
        for gateway in gateways_paginated:
            room = Room.query.filter_by(id=gateway.room_id).first()
            if room:
                zone = Zone.query.filter_by(zone_id=room.zone_id).first()
                if zone:
                    zone_name = zone.zone_name
                else:
                    zone_name = ''
                room_name = room.room_name
            else:
                zone_name = ''
                room_name = ''
            serialized_gateways.append({
                'id': gateway.gateway_id,
                'zone_name': zone_name,
                'room_name': room_name,
                'gateway_name': gateway.gateway_name,
                'gateway_mac_address': gateway.gateway_mac_address,
                'x_coordinate': gateway.x_coordinate,
                'y_coordinate': gateway.y_coordinate
            })
        if serialized_gateways:
            return make_response(jsonify({'gateways': serialized_gateways, 'data': data}), 200)
        else:
            return make_response(jsonify({ 'message': 'No Gateway found', 'data': data}), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', "status": "error"}), 200)
@gateways.route('/api/v1/get/gateway_by_id/<int:gateway_id>', methods=['GET'])
@token_required(1, 2, 3)
def get_gateway(current_user, gateway_id):
    try:
        gateway = single_record_query(Gateways, Gateways.gateway_id, gateway_id, current_user)
        if gateway:
            zone = Zone.query.filter_by(zone_id=gateway.zone_id).first()
            room = Room.query.filter_by(id=gateway.room_id).first()
            if zone:
                zone_name = zone.zone_name
            else:
                zone_name = ""
            if room:
                room_name = room.room_name
            else:
                room_name = ""
            return make_response(jsonify({'gateway': {
                'id': gateway.gateway_id,
                'zone_name': zone_name,
                'room_name': room_name,
                'gateway_name': gateway.gateway_name,
                'gateway_mac_address': gateway.gateway_mac_address,
                'x_coordinate': gateway.x_coordinate,
                'y_coordinate': gateway.y_coordinate,
            }}), 200)
        return make_response(jsonify({'message': 'Gateway not found', "status": "error"}), 404)
    except Exception as e:
        return make_response(jsonify({'message': f'error: {e}', "status": "error"}), 500)
@gateways.route('/api/v1/update/gateway/<int:gateway_id>', methods=['PUT'])
@token_required(1, 2, 3)
def update_gateway(current_user, gateway_id):
    try:
        gateway = single_record_query(Gateways, Gateways.gateway_id, gateway_id, current_user)
        if not gateway:
            return make_response(jsonify({"message": "Gateway not found", "status": "error"}), 404)
        gateway_dict = json.loads(request.form.get('formData'))
        for key, value in gateway_schema.load({
            'gateway_name': gateway_dict.get('gateway_name').strip(),
            'gateway_mac_address': gateway_dict.get('gateway_mac_address').strip(),
            'x_coordinate': gateway_dict.get('x_coordinate'),
            'y_coordinate': gateway_dict.get('y_coordinate'),
            'zone_id': gateway_dict.get('zone_id', gateway.zone_id),
            'room_id': gateway_dict.get('room_id', gateway.room_id),
            'room_name': gateway_dict.get('room_name'),
        }).items():
            setattr(gateway, key, value)
        db.session.commit()
        return make_response(jsonify({'message': 'Gateway updated successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to update  gateway', "status": "error"}), 500)
@gateways.route('/api/v1/delete/gateway/<int:id>', methods=['DELETE'])
@token_required(1, 2, 3)
def delete_gateway(current_user, id):
    return delete_record(Gateways, current_user, id, "Gateways", "gateway_id")